require 'opentsdb'
require 'telltale/data'

module Telltale
  class << self
    def setup!
      if report?
        configure_request_instrumentation
        Rails.logger.info "Telltale started, will report to #{opentsdb_host} on port #{opentsdb_port}"
      else
        Rails.logger.error 'TELLTALE_OPENTSDB_HOST not set. Telltale will not report to OpenTSDB.'
      end
    end

    def configure_request_instrumentation
      ActiveSupport::Notifications.subscribe 'process_action.action_controller' do |_name, started, finished, _unique_id, payload|
        next unless report?
        client.put Telltale::Data.compile(started, finished, payload)
      end
    end

    def report?
      opentsdb_host.present? && !Rails.env.test?
    end

    def opentsdb_port
      ENV['TELLTALE_OPENTSDB_PORT'] || 4242
    end

    def opentsdb_host
      ENV['TELLTALE_OPENTSDB_HOST']
    end

    def client
      OpenTSDB::Client.new(hostname: opentsdb_host, port: opentsdb_port)
    end
  end
end

require 'telltale/railtie'
