module Telltale
  class Data
    attr_reader :started, :finished, :payload

    def self.compile(started, finished, payload)
      new(started, finished, payload).all
    end

    def initialize(started, finished, payload)
      @started = started
      @finished = finished
      @payload = payload
    end

    def all
      [total_time, request_count]
    end

    def total_time
      {
        metric: "#{metric_prefix}.rails.total_time",
        value: ((finished - started) * 1000).ceil,
        timestamp: Time.now.to_i,
        tags: tags,
      }
    end

    def request_count
      {
        metric: "#{metric_prefix}.rails.request_count",
        value: 1,
        timestamp: Time.now.to_i,
        tags: tags,
      }
    end

    private

    def metric_prefix
      ENV['TELLTALE_PREFIX'] || 'telltale'
    end

    def tags
      {
        http_status: http_status,
        controller: payload[:controller],
        action: [payload[:controller], payload[:action]].join('.'),
        host: Socket.gethostname,
        environment: Rails.env,
        app: Rails.application.class.parent_name.downcase,
      }
    end

    def http_status
      return payload[:status] if payload[:status]
      return 500 if payload[:exception].present?
    end
  end
end
